@@ -305,7 +305,7 @@ meta_plot <- function(gt, x, legend_pos, bb, metaX, metaY, frameX, frameY, use_f
 				sortid=gt$logo.id,
 				stringsAsFactors = FALSE) else NULL,
 			if (gt$scale.show) data.frame(type="scale_bar",
-				height=3*lineHeight * gt$scale.size,
+				height=3*lineHeight * gt$scale.text.size,
 				width=1,
 				position1=gt$scale.position[1],
 				position2=gt$scale.position[2], 
@@ -314,8 +314,8 @@ meta_plot <- function(gt, x, legend_pos, bb, metaX, metaY, frameX, frameY, use_f
 				sortid=gt$scale.id,
 				stringsAsFactors = FALSE) else NULL,
 			if (gt$compass.show) data.frame(type="compass",
-				height=(gt$compass.nlines * gt$compass.fontsize)*lineHeight,
-				width=(gt$compass.nlines * gt$compass.fontsize)*lineWidth,
+				height=(gt$compass.nlines * gt$compass.text.size)*lineHeight,
+				width=(gt$compass.nlines * gt$compass.text.size)*lineWidth,
 				position1=gt$compass.position[1],
 				position2=gt$compass.position[2], 
 				just1=gt$compass.just[1],
