@@ -5,6 +5,9 @@
 		max.categories = 30,
 		max.raster = c(plot = 1e7, view = 1e6),
 		show.messages = TRUE,
+		output.format = "png",
+		output.size = 49,
+		output.dpi = 300,
 		title = NA,
 		scale = 1,
 		title.size = 1.3,
@@ -52,9 +55,11 @@
 		legend.hist.height = 0.3,
 		legend.hist.width = 0.4,
 		#legend.width,
+		legend.title.color = NULL,
 		legend.title.size = 1.1,
 		legend.title.fontface = NULL,
 		legend.title.fontfamily = NULL,
+		legend.text.color = NULL,
 		legend.text.size = 0.7,
 		legend.text.fontface = NULL,
 		legend.text.fontfamily = NULL,
@@ -63,6 +68,9 @@
 			fun = NULL,
 			scientific = FALSE,
 			digits = NA,
+			big.num.abbr = c(mln = 6, bln = 9),
+			prefix = "",
+			suffix = "",
 			text.separator = "to",
 			text.less.than = c("Less", "than"),
 			text.or.more = c("or", "more"),
@@ -71,7 +79,6 @@
 		),
 		legend.frame = FALSE,
 		legend.frame.lwd = 1,
-		legend.text.color = NULL,
 		legend.bg.color = NA,
 		legend.bg.alpha = 1,
 		legend.hist.bg.color = NA,
@@ -325,13 +332,16 @@
 #' @param qtm.scalebar should a scale bar be added to interactive maps created with \code{\link{qtm}}. In other words, should \code{tm_scale_bar()} be added automatically? The value \code{NA} means that the scale bar is only added when \code{\link{qtm}} is called without arguments or with a search term. The default value is \code{TRUE}.
 #' @param qtm.minimap should a minimap be added to interactive maps created with \code{\link{qtm}}. In other words, should \code{tm_minimap()} be added automatically? The value \code{NA} means that the minimap is only added in navigation mode (i.e. when \code{\link{qtm}} is called without arguments or with a search term. The default value is \code{FALSE}.
 #' @param show.messages should messages be shown?
+#' @param output.format The format of the static maps saved with \code{\link{tmap_save}} without specification of the filename. The default is \code{"png"}.
+#' @param output.size The size of the static maps saved with \code{\link{tmap_save}} without specification of width and height. The unit is squared inch and the default is 49. This means that square maps (so with aspect ratio 1) will be saved as 7 by 7 inch images and a map with aspect ratio 2 (e.g. most world maps) will be saved as approximately 10 by 5 inch.
+#' @param output.dpi The default number of dots per inch for \code{\link{tmap_save}} and \code{\link{tmap_animation}}.
 #' @param style style name
 #' @example ./examples/tmap_options.R
 #' @rdname tmap_options
 #' @name tmap_options
 #' @export
 #' @seealso \code{\link{tm_layout}}, \code{\link{tm_view}}, and \code{\link{tmap_style}}
-tmap_options <- function(..., unit, limits, max.categories, max.raster, basemaps, basemaps.alpha, overlays, overlays.alpha, qtm.scalebar, qtm.minimap, show.messages) {
+tmap_options <- function(..., unit, limits, max.categories, max.raster, basemaps, basemaps.alpha, overlays, overlays.alpha, qtm.scalebar, qtm.minimap, show.messages, output.format, output.size, output.dpi) {
 
 	.tmapOptions <- get(".tmapOptions", envir = .TMAP_CACHE)	
 	current.style <- getOption("tmap.style")
