@@ -1,9 +1,11 @@
-view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
+view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL, in.shiny = FALSE, lf = NULL) {
 	
 	gt <- gp$tm_layout
 	gp$tm_layout <- NULL
 	
-	lf <- leaflet(options = leaflet::leafletOptions(crs=gt$projection))
+	proxy <- !is.null(lf)
+	
+	if (!proxy) lf <- leaflet(options = leaflet::leafletOptions(crs=gt$projection))
 
 	pOptions <- function(cw) {
 		if (is.null(cw)) cw <- 20
@@ -18,11 +20,13 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 		
 	
 	# add background overlay
-	lf <- appendContent(lf, {
-		tags$head(
-			tags$style(HTML(paste(".leaflet-container {background:", gt$bg.color, ";}", sep="")))
-		)	
-	})
+	if (!in.shiny) {
+		lf <- appendContent(lf, {
+			tags$head(
+				tags$style(HTML(paste(".leaflet-container {background:", gt$bg.color, ";}", sep="")))
+			)	
+		})
+	}
 	
 	# if (gt$bg.overlay.alpha!=0) {
 	# 	if (any(sapply(gp, function(gpl)!is.null(gpl$raster)))) {
@@ -63,45 +67,28 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 	overlays_tiles <- character(0)
 	
 	## keep track of layerIds to prevent duplicates (duplicates are not shown in leaflet)
-	layerIds <- list(polygons = character(0),
-					 lines = character(0),
-					 points = character(0))
+	# layerIds <- list(polygons = character(0),
+	# 				 lines = character(0),
+	# 				 points = character(0))
 	
+	
+	if (proxy) {
+		layerIds <- get(".layerIdsNew", envir = .TMAP_CACHE)
+		if (length(layerIds) == 0) {
+			start_pane_id <- 401
+		} else {
+			start_pane_id <- min(as.integer(substr(names(layerIds), 5, 7)))	
+		}
+	} else {
+		layerIds <- list()
+		start_pane_id <- 401
+	}
+	
+
+
 	# should the layer control include base layers? TRUE if |basemaps| > 1 || names/groups are specified
 	basename.specified <- FALSE
 	
-	# workaround for https://github.com/rstudio/leaflet/issues/427
-	# pane <- "overlayPane"
-	# paneUsed <- FALSE
-	# nextPane <- function(pane) {
-	# 	pane <- "markerPane"  # when pane=overlayPane tiles are still below shadowPane
-	# 	# pane <- switch(pane,
-	# 	# 	  shadowPane = "overlayPane",
-	# 	# 	  "markerPane")
-	# 	assign("pane", pane, envir = e)
-	# 	assign("paneUsed", FALSE, envir = e)
-	# 	pane
-	# }
-	# usePane <- function() {
-	# 	assign("paneUsed", TRUE, envir = e)
-	# }
-	
-	
-	pane <- "overlayPane00"
-	paneID <- function(pane) as.integer(substr(pane, 12,14))
-	
-	nextPane <- function(pane) {
-		pane <- paste0("overlayPane", sprintf("%02d", paneID(pane) + 1))
-		assign("pane", pane, envir = e)
-		pane
-	} 
-	
-	
-	
-	addPane <- function(lf, pane) {
-		addMapPane(lf, pane, zIndex = paneID(pane) + 400)
-	}
-	
 
 	addBaseGroup <- function(group) {
 		for (g in group) {
@@ -124,16 +111,58 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 	}
 	
 	addOverlayGroup <- function(group, are.tiles = FALSE) {
-		if (is.na(overlays[1])) {
-			overlays <- group
-		} else if (!(group %in% overlays)) {
-			overlays <- c(overlays, group)
+		for (g in group) {
+			if (is.na(overlays[1])) {
+				overlays <- g
+			} else if (!(g %in% overlays)) {
+				overlays <- c(overlays, g)
+			}
 		}
 		assign("overlays", overlays, envir = e)
 		if (are.tiles) assign("overlays_tiles", c(overlays_tiles, group), envir = e)
 	}
 	
-	group_selection <- mapply(function(shp, gpl, shp_name) {
+	### find z indeces and create tmapXXX panes
+	zids <- lapply(gp, function(gpl) {
+		po <- gpl$plot.order
+		
+		po2 <- substr(po, 4, nchar(po))
+		po2[po2 == "symbols"] <- "symbol"
+		po2[po2 == "tiles"] <- "tile"
+		po2[po2 == "lines"] <- "line"
+
+		zi <- sapply(po2, function(p) {
+			if (p == "grid") gt$grid.zindex else gpl[[paste0(p, ".zindex")]]
+		})
+
+		if (!is.null(gpl$tile.gtype) && gpl$tile.gtype == "base") {
+			zi[names(zi) == "tile"] <- 0
+		}
+		zi
+	})
+	
+	zids_vec <- unlist(zids)
+	zids_vec[is.na(zids_vec)] <- seq(start_pane_id, length.out = sum(is.na(zids_vec)))
+	zids_len <- sapply(zids, length)
+	zindices <- split(zids_vec, unlist(mapply(rep, 1:length(zids), each = zids_len, SIMPLIFY = FALSE)))
+	tmap_zindices <- sort(unique(unname(setdiff(zids_vec, 0))))
+
+	## get/set existing panes
+	if (!proxy) {
+		assign("pane_ids", tmap_zindices, envir = .TMAP_CACHE)
+		z_panes <- integer()
+	} else {
+		z_panes <- get("pane_ids", envir = .TMAP_CACHE)
+		assign("pane_ids", union(tmap_zindices, z_panes), envir = .TMAP_CACHE)
+	}
+	
+	# add new panes
+	for (z in setdiff(tmap_zindices, z_panes)) {
+		lf <- addMapPane(lf, paneName(z), zIndex = z)
+	}
+	
+
+	group_selection <- mapply(function(shp, gpl, shp_name, zindex) {
 		if (!is.null(shp)) {
 			if (nrow(shp) == 0) {
 				shp <- NULL
@@ -154,7 +183,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 		}
 		
 		
-		plot_tm_fill <- function() {
+		plot_tm_fill <- function(zi) {
 			if (is.null(shp)) return(FALSE)
 			
 			bres <- split_alpha_channel(gpl$col, alpha=alpha)
@@ -173,9 +202,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 				labels <- NULL
 			}
 			
-			shp$tmapID <- if (!is.null(labels)) as.character(labels) else shp$tmapID
-			shp$tmapID2 <- submit_labels(shp$tmapID, "polygons", e)
-			
+			dashArray <- lty2dashArray(gpl$lty)
 			stroke <- gpl$lwd>0 && !is.na(bcol) && bopacity!=0
 			
 			charwidth <- attr(popups, "charwidth")
@@ -183,11 +210,15 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			group_name <- if (is.na(gpl$fill.group)) shp_name else gpl$fill.group
 			addOverlayGroup(group_name)
 			
-			pane <- nextPane(pane)
-			lf <- addPane(lf, pane)
+			pane <- paneName(zi)
 			
+			# pane <- nextPane(pane)
+			# lf <- addPane(lf, pane)
 			
-			lf <- lf %>% addPolygons(data=shp, label = ~tmapID, layerId = shp$tmapID2, stroke=stroke, weight=gpl$lwd, color=bcol, fillColor = fcol, opacity=bopacity, fillOpacity = fopacity, popup = popups, options = pathOptions(clickable=!is.null(popups), pane=pane), group=group_name, popupOptions = pOptions(charwidth))
+			shp$tmapID <- if (!is.null(labels)) as.character(labels) else shp$tmapID
+			shp$tmapID2 <- submit_labels(shp$tmapID, "polygons", pane, e)
+			
+			lf <- lf %>% addPolygons(data=shp, label = ~tmapID, layerId = shp$tmapID2, stroke=stroke, weight=gpl$lwd, color=bcol, fillColor = fcol, opacity=bopacity, fillOpacity = fopacity, dashArray = dashArray, popup = popups, options = pathOptions(clickable=!is.null(popups), pane=pane), group=group_name, popupOptions = pOptions(charwidth))
 			
 			# if (!is.null(labels)) {
 			# 	lf <- lf %>% 
@@ -195,14 +226,14 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			# }
 
 			if (!is.na(gpl$xfill[1])) {
-				if (gpl$fill.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="fill", alpha=alpha, group = if (gt$free.scales.fill) group_name else NULL)
+				if (gpl$fill.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="fill", alpha=alpha, group = if (gt$free.scales.fill) group_name else NULL, zindex = zi)
 			}
 
 			assign("lf", lf, envir = e)
 			TRUE
 		}
 		
-		plot_tm_lines <- function() {
+		plot_tm_lines <- function(zi) {
 			if (is.null(shp)) return(FALSE)
 			lres <- split_alpha_channel(gpl$line.col, alpha=alpha)
 			lcol <- lres$col
@@ -214,14 +245,18 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			
 			dashArray <- lty2dashArray(gpl$line.lty)
 			
-			shp$tmapID <- if (!is.null(labels)) as.character(labels) else shp$tmapID
-			shp$tmapID2 <- submit_labels(shp$tmapID, "lines", e)
 
 			group_name <- if (is.na(gpl$line.group)) shp_name else gpl$line.group
 			addOverlayGroup(group_name)
 			
-			pane <- nextPane(pane)
-			lf <- addPane(lf, pane)
+			pane <- paneName(zi)
+			# pane <- nextPane(pane)
+			# lf <- addPane(lf, pane)
+			
+			shp$tmapID <- if (!is.null(labels)) as.character(labels) else shp$tmapID
+			shp$tmapID2 <- submit_labels(shp$tmapID, "lines", pane, e)
+			
+			
 			lf <- lf %>% addPolylines(data=shp, label = ~tmapID, layerId = shp$tmapID2, stroke=TRUE, weight=gpl$line.lwd, color=lcol, opacity = lopacity, popup = popups, options = pathOptions(clickable=!is.null(popups), pane=pane), dashArray=dashArray, group=group_name, popupOptions = pOptions(charwidth)) 
 
 			# if (!is.null(labels)) {
@@ -230,7 +265,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			# }
 			
 			if (!is.na(gpl$xline[1])) {
-				if (gpl$line.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="line.col", alpha=alpha, group = if (gt$free.scales.line.lwd) group_name else NULL)
+				if (gpl$line.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="line.col", alpha=alpha, group = if (gt$free.scales.line.lwd) group_name else NULL, zindex = zi)
 			}
 			
 			if (!is.na(gpl$xlinelwd[1]) && gpl$line.lwd.legend.show) {
@@ -247,7 +282,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			# 									   lineend="butt"), i, k)
 		}
 		
-		plot_tm_symbols <- function() {
+		plot_tm_symbols <- function(zi) {
 			if (is.null(shp)) return(FALSE)
 			npol <- nrow(co)
 			
@@ -284,10 +319,15 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 					gpl$symbol.names <- "MARKER__TEXT"
 				}
 			}
+
+			pane <- paneName(zi)
+			# pane <- nextPane(pane)
+			# lf <- addPane(lf, pane)
 			
+						
 			popups <- get_popups(gpl, type="symbol")
 			labels <- as.character(get_labels(gpl, type="symbol"))
-			ids <- submit_labels(labels, "points", e)
+			ids <- submit_labels(labels, "symbols", pane, e)
 			
 			
 			charwidth <- attr(popups, "charwidth")
@@ -325,8 +365,6 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			group_name <- if (is.na(gpl$symbol.group)) shp_name else gpl$symbol.group
 			addOverlayGroup(group_name)
 			
-			pane <- nextPane(pane)
-			lf <- addPane(lf, pane)
 			
 			if (are.icons) {
 				#symbol.size2 <- symbol.size2 / 3 # Correct for the fact that markers are larger than circle markers. This is good, but for static plots the icon size was already increased by icon.size=3, so this is to revert it for view mode
@@ -363,7 +401,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			# }
 			
 			if (!is.na(gpl$xcol[1])) {
-				if (gpl$symbol.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="symbol.col", alpha=alpha, group = if (gt$free.scales.symbol.col) group_name else NULL)
+				if (gpl$symbol.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="symbol.col", alpha=alpha, group = if (gt$free.scales.symbol.col) group_name else NULL, zindex = zi)
 			}
 			
 			if (!is.na(gpl$xsize[1]) && gpl$symbol.size.legend.show) {
@@ -377,7 +415,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			TRUE
 			
 		}
-		plot_tm_text <- function() {
+		plot_tm_text <- function(zi) {
 			if (is.null(shp)) return(FALSE)
 # 			if (is.null(gpl$symbol.misc) || !gpl$symbol.misc$symbol.are.markers) {
 # 				warns["text"] <- TRUE
@@ -421,8 +459,9 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			group_name <- if (is.na(gpl$text.group)) shp_name else gpl$text.group
 			addOverlayGroup(group_name)
 			
-			pane <- nextPane(pane)
-			lf <- addPane(lf, pane)
+			pane <- paneName(zi)
+			# pane <- nextPane(pane)
+			# lf <- addPane(lf, pane)
 			
 			
 			if (length(cs_set)==1) {
@@ -448,14 +487,14 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			}
 			
 			if (!is.na(gpl$xtcol[1])) {
-				if (gpl$text.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="text.col", alpha=alpha, group = if (gt$free.scales.text.col) group_name else NULL)
+				if (gpl$text.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="text.col", alpha=alpha, group = if (gt$free.scales.text.col) group_name else NULL, zindex = zi)
 			}
 			
 			assign("lf", lf, envir = e)
 
 			TRUE
 		}
-		plot_tm_raster <- function() {
+		plot_tm_raster <- function(zi) {
 			if (is.null(shp)) return(FALSE)
 			if (gpl$raster.misc$is.OSM) {
 				if (is.na(gpl$raster.misc$leaflet.server)) {
@@ -499,23 +538,24 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 			lf <- lf %>% addRasterImage(x=shp, colors=mappal, opacity = pal_opacity, group=group_name, project = FALSE)
 			
 			if (!is.na(gpl$xraster[1])) {
-				if (gpl$raster.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="raster", alpha=alpha, group = if (gt$free.scales.raster) group_name else NULL)
+				if (gpl$raster.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="raster", alpha=alpha, group = if (gt$free.scales.raster) group_name else NULL, zindex = zi)
 			}
 
 			assign("lf", lf, envir = e)
 			TRUE
 		}
-		plot_tm_grid <- function() {
-			lf <- lf %>% addGraticule()
+		plot_tm_grid <- function(zi) {
+			lf <- lf %>% addGraticule(options = pathOptions(pane = paneName(zi)))
 			
 			assign("lf", lf, envir = e)
 			TRUE
 		}
 		
-		plot_tm_tiles <- function() {
+		plot_tm_tiles <- function(zi) {
 			basemaps <- gpl$tile.server
 			basemaps.alpha <- gpl$tile.alpha
-			type <- gpl$tile.grouptype
+			type <- gpl$tile.gtype
+			tms <- gpl$tile.tms
 			
 			if (is.null(basemaps)) {
 				return(FALSE)
@@ -529,7 +569,9 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 
 			
 			
-			group_names <- if (is.na(gpl$tile.group[1])) {
+			group_names <- if (is.null(gpl$tile.group)) {
+				NULL
+			} else if (is.na(gpl$tile.group[1])) {
 				vapply(basemaps, FUN = function(bm) {
 					if (substr(bm, 1, 4) == "http") {
 						x <- strsplit(bm, "/", fixed=TRUE)[[1]]
@@ -543,29 +585,31 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 				rep(gpl$tile.group, length.out = length(basemaps))
 			}
 			
-			
-			if (type == "base") {
-				addBaseGroup(group_names)	
-			} else {
-				addOverlayGroup(group_names, are.tiles = TRUE)
+			if (!is.null(group_names)) {
+				if (type == "base") {
+					addBaseGroup(group_names)	
+				} else {
+					addOverlayGroup(group_names, are.tiles = TRUE)
+				}
 			}
 			
 			if(type == "base") {
 				pane <- "tilePane"
 			} else {
-				pane <- nextPane(pane)
-				lf <- addPane(lf, pane)
+				pane <- paneName(zi)
+				# pane <- nextPane(pane)
+				# lf <- addPane(lf, pane)
 			}
 				
 			if (!is.na(gt$set.zoom.limits[1])) {
-				tileOptions <- lapply(basemaps.alpha, function(a) {
-					tileOptions(minZoom=gt$set.zoom.limits[1], maxZoom=gt$set.zoom.limits[2], opacity=a, pane=pane)
-				})
+				tileOptions <- mapply(function(a, tmsi) {
+					tileOptions(minZoom=gt$set.zoom.limits[1], maxZoom=gt$set.zoom.limits[2], opacity=a, pane=pane, tms = tmsi)
+				}, basemaps.alpha, tms, SIMPLIFY = FALSE)
 				
 			} else {
-				tileOptions <- lapply(basemaps.alpha, function(a) {
-					tileOptions(opacity=a, pane=pane)
-				})
+				tileOptions <- mapply(function(a, tmsi) {
+					tileOptions(opacity=a, pane=pane, tms = tmsi)
+				}, basemaps.alpha, tms, SIMPLIFY = FALSE)
 			}
 			
 			allLettersOrDots <- function(x) grepl("^[A-Za-z\\.]*$", x)
@@ -575,7 +619,8 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 				for (i in 1:length(basemaps)) {
 					bm <- unname(basemaps[i])
 					bmname <- unname(group_names[i])
-					if (allLettersOrDots(substr(bm, 1, 4))) {
+					bm4 <- substr(bm, 1, 4)
+					if (allLettersOrDots(bm4) && bm4 != "http") {
 						# it is not possible to check provider options, since leaflet::providers is not exported
 						lf <- lf %>% addProviderTiles(bm, group=bmname, options = tileOptions[[i]])
 					} else {
@@ -591,9 +636,13 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 		e2 <- environment()
 		
 		fnames <- paste("plot", gpl$plot.order, sep="_")
-		layer_selection <- vapply(fnames, do.call, logical(1), args=list(), envir=e2)
+
+		layer_selection <- unlist(mapply(function(fn, zi) {
+			do.call(fn, list(zi = zi), envir = e2)
+		}, fnames, zindex, SIMPLIFY = FALSE))
+			
 		any(layer_selection)
-	}, shps, gp, gt$shp_name, SIMPLIFY = TRUE)
+	}, shps, gp, gt$shp_name, zindices, SIMPLIFY = TRUE)
 	
 	if (gt$show.messages && showWarns) {
 		if (warns["symbol"]) message("Symbol shapes other than circles or icons are not supported in view mode.")
@@ -619,6 +668,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 				opacity <- unname(RGBA[4,1]/255) * alpha
 				
 				lf <- lf %>% addLegend(position=gt$view.legend.position,
+									   group = gali$group,
 									   colors = col,
 									   labels = gali$labels,
 									   title=gali$title, 
@@ -712,9 +762,13 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 	
 	
 
-	lf <- set_bounds_view(lf, gt)
+	if (!proxy) lf <- set_bounds_view(lf, gt)
 	lf$title <- gt$title
-	lf
+	
+	assign(".layerIds", layerIds, envir = .TMAP_CACHE)
+	
+	lf	
+	
 }
 
 set_bounds_view <- function(lf, gt) {
@@ -802,14 +856,15 @@ format_popups <- function(id=NULL, titles, format, values) {
 	
 	nc_labels <- nchar(labels)
 	nc_titles <- max(nchar(titles_format))
-	nc_values_format <- do.call(pmax, lapply(values_format, nchar)) + 2 # which space between title and value
-	
+	nc_values_format <- do.call(pmax, c(lapply(values_format, nchar, allowNA = TRUE), list(na.rm = TRUE))) + 2 # which space between title and value
+
 	charwidth <- max(nc_labels + nc_titles + nc_values_format)
 	# print(which.max(nc_labels + nc_titles + nc_values_format))
 	attr(x, "charwidth") <- charwidth
 	x
 }
 
+
 split_alpha_channel <- function(x, alpha) {
 	if (is.null(x)) {
 		list(col=NULL, opacity=0)
@@ -868,7 +923,7 @@ get_popups <- function(gpl, type) {
 	popups
 }
 
-add_legend <- function(map, gpl, gt, aes, alpha, group, list.only=FALSE) {
+add_legend <- function(map, gpl, gt, aes, alpha, group, list.only=FALSE, zindex = NULL) {
 	pal_name <- paste(aes, "legend.palette", sep=".")
 	val_name <- paste(aes, "legend.values", sep=".")
 	lab_name <- paste(aes, "legend.labels", sep=".")
@@ -877,6 +932,12 @@ add_legend <- function(map, gpl, gt, aes, alpha, group, list.only=FALSE) {
 	val <- gpl[[val_name]]
 	lab <- gpl[[lab_name]]
 	
+	if (!is.null(zindex)) {
+		layerId <- legendName(zindex)
+	} else {
+		layerId <- NULL
+	}
+	
 	if (nchar(pal[1])>10) {
 		# check whether style is continuous
 		style <- attr(pal, "style")
@@ -942,19 +1003,21 @@ add_legend <- function(map, gpl, gt, aes, alpha, group, list.only=FALSE) {
 
 	legend.position <-gt$view.legend.position
 
+	
+
 	if (is.cont) {
 		legvals <- if (!is.na(colNA)) c(val, NA) else val
 
 		if (style=="quantile") {
 			addLegend(map, position=legend.position, group = group,
-					  pal=colorQuantile(col, val, na.color=colNA, alpha = FALSE), values=legvals, na.label = textNA, title=title, opacity=opacity)
+					  pal=colorQuantile(col, val, na.color=colNA, alpha = FALSE), values=legvals, na.label = textNA, title=title, opacity=opacity, layerId = layerId)
 		} else {
 			addLegend(map, position=legend.position, group = group,
-					  pal=colorNumeric(col, val, na.color=colNA, alpha = FALSE), values=legvals, na.label = textNA, title=title, opacity=opacity)
+					  pal=colorNumeric(col, val, na.color=colNA, alpha = FALSE), values=legvals, na.label = textNA, title=title, opacity=opacity, layerId = layerId)
 		}
 	} else {
 		if (allNAs) {
-			addLegend(map, position=legend.position, group = group, colors=colNA, labels=textNA, title=title, opacity=opacity)
+			addLegend(map, position=legend.position, group = group, colors=colNA, labels=textNA, title=title, opacity=opacity, layerId = layerId)
 		} else {
 			if (!is.na(colNA)) {
 				legvals <- c(lab, textNA)
@@ -963,10 +1026,12 @@ add_legend <- function(map, gpl, gt, aes, alpha, group, list.only=FALSE) {
 				legvals <- lab
 			}
 			addLegend(map, position=legend.position,
+					  group = group,
 					  colors = col,
 					  labels = legvals,
 					  title=title,
-					  opacity=opacity)
+					  opacity=opacity,
+					  layerId = layerId)
 			
 		}
 	}
@@ -983,12 +1048,12 @@ working_internet <- function(url = "http://www.google.com") {
 }
 
 bbx_per_line <- function(bbx) {
-	max_lines <- 60 #par("din")[2]*10 #disabled because window height doesn't influence scaling
+	max_lines <- 60
 	(bbx[4] - bbx[2]) / max_lines
 }
 
 units_per_line <- function(bbx) {
-	max_lines <- 60 #par("din")[2]*10 #disabled because window height doesn't influence scaling
+	max_lines <- 60
 
 	# calculate top-center to bottom-center
 	vdist <- tmaptools::approx_distances(bbx, projection = "longlat", target = "m")$vdist
@@ -1027,21 +1092,55 @@ get_epsg_number <- function(proj) {
 	if (length(epsg)==0) NA else epsg
 }
 
-submit_labels <- function(labels, cls, e) {
+# submit_labels <- function(labels, cls, e) {
+# 	
+# 	layerIds <- get("layerIds", envir = e)
+# 
+# 	labels_all <- layerIds[[cls]]
+# 	
+# 	pos <- length(labels_all)
+# 	
+# 	labels_all <- make.names(c(labels_all, labels), unique = TRUE)
+# 	
+# 	labels <- labels_all[(pos + 1): length(labels_all)]	
+# 
+# 	layerIds[[cls]] <- labels_all
+# 	assign("layerIds", layerIds, envir = e)
+# 	labels
+# }
+
+submit_labels <- function(labels, cls, pane, e) {
 	
 	layerIds <- get("layerIds", envir = e)
-
-	labels_all <- layerIds[[cls]]
+	
+	
+	types <- attr(layerIds, "types")
+	
+	labels_all <- unlist(layerIds)
 	
 	pos <- length(labels_all)
 	
 	labels_all <- make.names(c(labels_all, labels), unique = TRUE)
 	
 	labels <- labels_all[(pos + 1): length(labels_all)]	
-
-	layerIds[[cls]] <- labels_all
+	
+	labelsList <- list(labels)
+	names(labelsList) <- pane
+	
+	layerIds <- c(layerIds, labelsList)
+	
+	#layerIds[[cls]] <- labels_all
+	
+	attr(layerIds, "types") <- c(types, cls)
+	
 	assign("layerIds", layerIds, envir = e)
 	labels
 }
 
+paneName <- function(x) {
+	paste0("tmap", sprintf("%03d", x))
+}
 
+legendName <- function(x) {
+	paste0("legend", sprintf("%03d", x))
+}
